# CollectX API Read

`clx_api.h` contains API for reading and accessing counters data from binary files written by CollectX.
`clx_counter_api_read_example.c` shows how to read the data from example binary file and the schema folder.

## API

| Syntax      | Description                        |
| ----------- | ---------------------------------- |
| `void* clx_api_read_create_context(char* schema_dir)`                   | Creates and returns application context. Reads all counter schemas from directory `schema_dir`. Returns NULL on error. |
| `void* clx_api_open_counter_file(void* context, char* filename)`   | Opens counter binary file, and returns file meta info. If schema for that file was not found in `schema_dir` previously, tries to find schema from the binary file folder. Returns NULL on error. |
| `void* clx_api_create_counterset(void* context, void* file)`       | Creates and returns empty counter set for current `file`. Returns NULL on error. |
| `void* clx_api_allocate_counters_buffer(void* cset)`               | Allocates and returns pointer to a buffer. Buffer size depends on number and types of counters included in `cset`. Returns NULL on error. |
| `int clx_api_get_next_data(void* file, void* cset, uint64_t* timestamp, void* data)` | Reads single block of counters data from `file` into buffer `data`. Writes to `buffer` only counters from `cset`. Returns: 1 if no more data to read / cannot read the next data block, 0 if not finished. |
| `clx_api_counter_info_t * clx_api_get_all_counters(void* context, void* file, int* num_counters)` | Returns all the counters of schema corresponding to input `file`. Fills `num_counters`. |
| `clx_api_counter_info_t * clx_api_get_counters(void* context, void* file, int* num_counters)` |  Returns all the counters included to the counter set `cset`. |
| `int clx_api_get_num_counters(void* cset)`                         | Returns number of counters contained by counter set `cset`. |
| `int clx_api_add_counter_exact(void* cset, char* name)`            | Adds a counter matching exact `name` to counter set `cset`, if it is not already included. Returns 1 on success or -1 on failure. |
| `int clx_api_add_counters_matching(void* cset, char* tok)`         | Adds all the counters matching token `tok` to counter set `cset`, that were not included previously. `tok` is a name fragment or a string in format "tok1+tok2-tok". In the last example, considers only counter names that include both "tok1" and "tok2", and not include "tok3". To match counter name from the begining use "^tok", for name end use "tok$", otherwise it will look for matches everywhere.   Returns 1 on success or -1 on failure. |
| `int clx_api_add_all_counters(void* cset)`                         | Adds all counters to counter set `cset` from the corresponding schema. Returns 1 on success or -1 on failure.|
| `clx_api_data_type_t clx_api_get_type(void* cset, unsigned idx)`   | Returns type of the `idx`-th counter of the counter set `cset`. Available types are CLX_API_DATA_TYPE_UINT64, CLX_API_DATA_TYPE_FP64, CLX_API_DATA_TYPE_BIT64, CLX_API_DATA_TYPE_INT64. |
| `double   clx_api_get_double(void* cset, unsigned idx, void* data)` | Reads and returns double value of `idx`-th counter of counter set `cset` from buffer `data`. |
| `int64_t  clx_api_get_int64(void* cset, unsigned idx, void* data)` | Reads and returns int64_t value of `idx`-th counter of counter set `cset` from buffer `data`.  |
| `uint64_t clx_api_get_uint64(void* cset, unsigned idx, void* data)` | Reads and returns uint64_t value of "idx"-th counter of counter set `cset` from buffer *data*. |
| `void clx_api_read_destroy_context(void* context)`                      | Application context destructor. |
| `void clx_api_destroy_counter_set(void* cset)`                     | Counter set destructor. Apply on each counter set. |
| `void clx_api_destroy_and_close_file(void* file)`                  | File meta info destructor. Apply on each file. |


## Workflow summary

Workflow example is shown in `clx_counter_api_read_example.c`.
Workflow is summarized below:

1. Initialize context, read schemas folder and file information
	1. Create context and load the schemas with `clx_api_read_create_context`.
	2. Open a binary file(s) with with `clx_api_open_counter_file`. If it cannot find schema in cache, it will look for it in bin-file folder.
2. Create counter set for each file and add counters of interest to it.
	1. initialize counterset with *cset = `clx_api_create_counterset(context, file)`
	2. Optionaly add:
		1. All counters from schema with `clx_api_add_all_counter(cset)`
		2. A single counter matching exact name with `clx_api_add_counter_exact(cset, exact_name)`
		3. A set of counters matching token with `clx_api_add_counters_matching(cset, token)`
	3. Get list of all available counters with `clx_api_get_all_counters(context, file, *num_counters)` or get list of counterset counters with `clx_api_get_counters(cset)`.
3. Access the data:
	1. Create data buffer for counter values with `data = clx_api_allocate_counters_buffer(cset)`. Buffer capacity is derived from counter set
	2. Read counters data from file to buffer by calling `done = clx_api_get_next_data(file, cset, timestamp, data)` in a loop. On the end of a file `done` will be 1.
	3. To receive the data go through the data buffer, check each value type with `clx_api_get_type(cset, idx)` against all available types.
4. Cleanup the memory by calling destructors for counter set, file, and context context. Free data buffer.

Note that multiple files can be loaded using the same context. Counter set must be created for each binary file.
