# clxpy

clxpy is a C++ extension module for python to handle CollectX python data.

## Prerequisites

- python3.6+
- clxapi package installed

## Usage
- ```
  export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/opt/mellanox/collectx/lib
  ```
   - Required for `clxpy.so` dependencies
- add `/opt/mellanox/collectx/lib` to `PYTHONPATH`. 2 options:
  1.  Run `python` from `/opt/mellanox/collectx/lib`
  2.
      ```python
      # in python script
      import sys
      sys.path.append("/opt/mellanox/collectx/lib")
      # clxpy is available
      import clxpy
      ```
    See examples in `/opt/mellanox/collectx/examples/clxpy`
## Documentation
clxpy does not support sub-interpreters and is not thread safe.  
Exceptions might be raised in cases of invalid parameters or runtime errors.
### get_data
`clxpy.get_data(data_dir : str="", file : str="", schemas_dir : str="", app_id : str="") -> dict`
##### Description
Read binary files of Collectx python data. Returns a `dict` where each key is a file name and value is a list with the parsed data.
##### Parameters
* `data_dir` - ***optional argument*** `str` Directory which contains schemas directory and binary files.
  * Function will search for one directory named '**schema**' and all files with '**.bin**' suffix.
  * **Note**: Multiple schema directories will cause undefined behavior.
* `file` - ***optional argument*** `str` Binary file name with collectx python data. If used, `schemas_dir` argument is required. Ignored if passed with `data_dir`.

* `schemas_dir` - ***optional argument*** `str` Directory name with schemas files. Should be passed together with `file`.
* `app_id` - ***optional argument*** `str` Used as data filter. If passed to function, the returned output will be for the specified app ID only. If not passed, output will contain all data in file(s).
##### Example
```python
import clxpy

DATA_DIR    = "/data"
SCHEMAS_DIR = "/data/schema"
FILE        = "/data/2022/0101/node/dict_tag_1649835828278848.bin"

d1 = clxpy.get_data(file=FILE, ​schemas_dir=SCHEMAS_DIR)

d2 = clxpy.get_data(data_dir=DATA_DIR)
# d2 contains at least the same data d1 has, since DATA_DIR contains FILE

d3 = clxpy.get_data(DATA_DIR)
# d3 and d2 have the same data
```

#
### send_to_fluentbit
`clxpy.send_to_fluentbit(data, name="forward", host="localhost", port="24224", params={}) -> None`  
##### Description
Send python object to a fluentbit endpoint. When configuration is changed, current connection will close to allow a new endpoint. If the configuration arguments are the same between calls, or no arguments passed to function, the existing fluentbit endpoint is used. The data sent as is, that is, without serialization. The same structure `data` argument has will be in the sent message.  
**Note**: An invalid fluentbit endpoint will cause undefined behaviour.
##### Parameters
- `data` ***required argument*** ​Following types are supported:  
   - `str`
   - `bool`
   - `int`
   - `float`
   - `dict`
  - `list`*  
    *Directly nested lists are not allowed since each object should be named. For example:
     ```python
     ​# in case of a dict
     ​valid_dict   = {"a": [1, True, 3.14]}    # OK - list has name "a"
     ​invalid_dict = {"a": [1, [True, 3.14]]}  # invalid - nested list is not named
     ​# in case of a list - the outermost list is always unnamed, but all inner lists must be named
     ​​valid_list   = [{"a": [1, True, 3.14]}]    # ok
     ​invalid_list = [{"a": [1, [True, 3.14]]}]  # invalid
- `name` - ***optional argument*** `str` Plugin name (default: `"forward"`).
- `host` - ***optional argument*** `str` Plugin host IP (default: `"localhost"`).
- `port` - ***optional argument*** `str` Plugin host port (default: `"24224"`).
- `params` - ***optional argument*** `dict` with `str` types that contains plugin parameters (default: `{}`).
